@@ -30,12 +30,12 @@ from oslo_config import cfg
 from oslo_log import log
 import oslo_messaging
 from oslo_utils import timeutils
-import six
-from six.moves.urllib import parse as urlparse
 from stevedore import extension
 from tooz import coordination
+from urllib import parse as urlparse
 
 from ceilometer import agent
+from ceilometer import cache_utils
 from ceilometer import declarative
 from ceilometer import keystone_client
 from ceilometer import messaging
@@ -65,7 +65,18 @@ POLLING_OPTS = [
     cfg.MultiStrOpt('pollsters_definitions_dirs',
                     default=["/etc/ceilometer/pollsters.d"],
                     help="List of directories with YAML files used "
-                         "to created pollsters.")
+                         "to created pollsters."),
+    cfg.BoolOpt('tenant_name_discovery',
+                default=False,
+                help='Identify project and user names from polled samples. '
+                     'By default, collecting these values is disabled due '
+                     'to the fact that it could overwhelm keystone service'
+                     'with lots of continuous requests depending upon the '
+                     'number of projects, users and samples polled from '
+                     'the environment. While using this feature, it is '
+                     'recommended that ceilometer be configured with a '
+                     'caching backend to reduce the number of calls '
+                     'made to keystone.'),
 ]
 
 
@@ -101,7 +112,7 @@ class Resources(object):
                     not self.agent_manager.partition_coordinator or
                     self.agent_manager.hashrings[
                         static_resources_group].belongs_to_self(
-                        six.text_type(v))] + source_discovery
+                        str(v))] + source_discovery
 
         return source_discovery
 
@@ -139,6 +150,10 @@ class PollingTask(object):
 
         self._telemetry_secret = self.manager.conf.publisher.telemetry_secret
 
+        self.ks_client = self.manager.keystone
+
+        self._cache = cache_utils.get_client(self.manager.conf)
+
     def add(self, pollster, source):
         self.pollster_matches[source.name].add(pollster)
         key = Resources.key(source.name, pollster)
@@ -195,6 +210,30 @@ class PollingTask(object):
                     for sample in samples:
                         # Note(yuywz): Unify the timestamp of polled samples
                         sample.set_timestamp(polling_timestamp)
+
+                        if (
+                            self.manager.conf.polling.tenant_name_discovery and
+                            self._cache
+                        ):
+
+                            # Try to resolve project UUIDs from cache first,
+                            # and then keystone
+                            if sample.project_id:
+                                sample.project_name = \
+                                    self._cache.resolve_uuid_from_cache(
+                                        "projects",
+                                        sample.project_id
+                                    )
+
+                            # Try to resolve user UUIDs from cache first,
+                            # and then keystone
+                            if sample.user_id:
+                                sample.user_name = \
+                                    self._cache.resolve_uuid_from_cache(
+                                        "users",
+                                        sample.user_id
+                                    )
+
                         sample_dict = (
                             publisher_utils.meter_message_from_counter(
                                 sample, self._telemetry_secret
@@ -210,6 +249,9 @@ class PollingTask(object):
                     if sample_batch:
                         self._send_notification(sample_batch)
 
+                    LOG.info("Finished polling pollster %(poll)s in the "
+                             "context of %(src)s", dict(poll=pollster.name,
+                                                        src=source_name))
                 except plugin_base.PollsterPermanentError as err:
                     LOG.error(
                         'Prevent pollster %(name)s from '
@@ -254,7 +296,8 @@ class AgentManager(cotyledon.Service):
                          for namespace in namespaces)
 
         # Create dynamic pollsters
-        extensions_dynamic_pollsters = self.create_dynamic_pollsters()
+        extensions_dynamic_pollsters = self.create_dynamic_pollsters(
+            namespaces)
 
         self.extensions = list(itertools.chain(*list(extensions))) + list(
             itertools.chain(*list(extensions_fb))) + list(
@@ -292,15 +335,18 @@ class AgentManager(cotyledon.Service):
         self._keystone = None
         self._keystone_last_exception = None
 
-    def create_dynamic_pollsters(self):
+    def create_dynamic_pollsters(self, namespaces):
         """Creates dynamic pollsters
 
         This method Creates dynamic pollsters based on configurations placed on
         'pollsters_definitions_dirs'
 
+        :param namespaces: The namespaces we are running on to validate if
+                           the pollster should be instantiated or not.
         :return: a list with the dynamic pollsters defined by the operator.
         """
 
+        namespaces_set = set(namespaces)
         pollsters_definitions_dirs = self.conf.pollsters_definitions_dirs
         if not pollsters_definitions_dirs:
             LOG.info("Variable 'pollsters_definitions_dirs' not defined.")
@@ -334,6 +380,21 @@ class AgentManager(cotyledon.Service):
 
             for pollster_cfg in pollsters_cfg:
                 pollster_name = pollster_cfg['name']
+                pollster_namespaces = pollster_cfg.get(
+                    'namespaces', ['central'])
+                if isinstance(pollster_namespaces, list):
+                    pollster_namespaces = set(pollster_namespaces)
+                else:
+                    pollster_namespaces = {pollster_namespaces}
+
+                if not bool(namespaces_set & pollster_namespaces):
+                    LOG.info("The pollster [%s] is not configured to run in "
+                             "these namespaces %s, the configured namespaces "
+                             "for this pollster are %s. Therefore, we are "
+                             "skipping it.", pollster_name, namespaces_set,
+                             pollster_namespaces)
+                    continue
+
                 if pollster_name not in pollsters_definitions:
                     LOG.info("Loading dynamic pollster [%s] from file [%s].",
                              pollster_name, pollsters_definitions_file)
@@ -550,7 +611,7 @@ class AgentManager(cotyledon.Service):
                         discovered = [
                             v for v in discovered if self.hashrings[
                                 self.construct_group_id(discoverer.group_id)
-                            ].belongs_to_self(six.text_type(v))]
+                            ].belongs_to_self(str(v))]
 
                     resources.extend(discovered)
                     if discovery_cache is not None:
